searchNodes=[{"doc":"Structured and declarative way to parse and validate parameters. Para uses Ecto under the hood and therefore inherits most of its utilities such as changeset and built-in validators. Usage Let's imagine that you have a controller named Web.UserController and wanted to validate the parameters for its :create and :update actions. First, let's define your parameters schema. defmodule Web.UserParams do use Para validator :create do required :name , :string required :age , :integer required :email , :string optional :phone , :string end validator :update do required :name , :string required :age , :integer required :email , :string optional :phone , :string end end This will generate two validate/2 functions for your module with action name and params as arguments. defmodule Web.UserController do use Web , :controller alias Web.UserParams , as : Params def create ( conn , params ) do with { :ok , data } &lt;- Params . validate ( :create , params ) do # ... end end def update ( conn , params ) do with { :ok , data } &lt;- Params . validate ( :update , params ) do # ... end end end The validate/2 function will return either an {:ok, map} or {:error, changeset} tuple. Inline validators Inline validator is a convenient way to validate your fields. This is especially useful when you need to perform some basic validation using Ecto.Changeset 's built-in validators. defmodule UserParams do use Para validator :update do required :name , :string , validator : { :validate_length , [ min : 3 , max : 100 ] } end end You can also use custom inline validators by supplying the function name as an atom. Similar to most of Ecto's built-in validators, the function will receive changeset , key , and opts as the arguments. defmodule UserParams do use Para validator :update do required :age , :string , validator : :validate_age required :gender , :string , validator : { :validate_gender , [ allow : :non_binary ] } end def validate_age ( changeset , key , opts ) do # ... end end Callback validator Sometimes, you might want to use custom validators or need to perform additional data manipulations. For this, you can use the callback/1 macro. The callback/1 macro will always be the last function to be called after the parametes have been parsed and validated. defmodule Web.UserParams do use Para validator :create do required :name , :string required :age , :integer required :email , :string optional :phone , :string callback :create_validators end def create_validators ( changeset , params ) do changeset |&gt; format_email ( params ) |&gt; format_phone ( params ) |&gt; validate_age ( ) end def format_email ( changeset , params ) do # ... end def format_phone ( changeset , params ) do # ... end def validate_age ( changeset ) do # ... end end","ref":"Para.html","title":"Para","type":"behaviour"},{"doc":"Define a custom callback function that will be called to perform any additional manipulation to the changeset or parameters. The callback function must accept two arguments namely changeset and params and return an Ecto.Changeset struct. Examples # Define callback function to be called validator :create do callback :validate_price end def validate_price ( changeset , params ) do #... end","ref":"Para.html#callback/1","title":"Para.callback/1","type":"macro"},{"doc":"Define an embedded array of maps field. It accepts similar schema definition like validator/2 . Examples defmodule OrderParams do use Para validator :create do embeds_many :items do required :title required :price , :float end end end","ref":"Para.html#embeds_many/2","title":"Para.embeds_many/2","type":"macro"},{"doc":"Define an embedded map field It accepts similar schema definition like validator/2 . Examples defmodule ParentParams do use Para validator :create do embeds_one :child do optional :name , :string optional :age , :integer end end end","ref":"Para.html#embeds_one/2","title":"Para.embeds_one/2","type":"macro"},{"doc":"Define an optional field. Please refer to required/3 for the list of available options.","ref":"Para.html#optional/3","title":"Para.optional/3","type":"macro"},{"doc":"Define a required field. Options :default - Assign a default value if the not set by input parameters :validator - Define either one of the built-in Ecto.Changeset's validators or use your own custom inline validator. Refer: Custom inline validator :droppable - Drop the field when the key doesn't exist in parameters. This is useful when you need to perform partial update by leaving out certain fields. Custom inline validator You can define your own validator as such: def validate_country ( changeset , field ) do # ... end Then use it as an inline validator for your field validator :create do required :country , :string , [ validator : :validate_country ] end You can also supply options with your custom inline validator validator :create do required :country , :string , [ validator : { :validate_country , region : :asia } ] end","ref":"Para.html#required/3","title":"Para.required/3","type":"macro"},{"doc":"Returns a basic spec. This is useful when you need to build a custom changeset, or when you just need the basic structure of your schema. Also see: Ecto.Changeset.change/2 Examples defmodule OrderParams do use Para validator :create do required :title required :data , { :array , :map } end end def changeset ( :new , params ) do spec = spec ( :create , params ) Ecto.Changeset . change ( spec . data , spec . types ) end","ref":"Para.html#c:spec/2","title":"Para.spec/2","type":"callback"},{"doc":"Parse and validate parameters for a given action. The function will cast all the returned map keys into atoms except for embedded map or list. Examples defmodule OrderParams do use Para validator :create do required :title required :data , { :array , :map } end end # Validate action with parameters OrderParams . validate ( :create , %{ &quot;title&quot; =&gt; &quot;test&quot; &quot;data&quot; =&gt; [ %{ &quot;color&quot; =&gt; &quot;black&quot; , &quot;material&quot; =&gt; &quot;cotton&quot; } ] } ) #=&gt; {:ok, %{ title : &quot;test&quot; data : [ %{ &quot;color&quot; =&gt; &quot;black&quot; , &quot;material&quot; =&gt; &quot;cotton&quot; } ] } }","ref":"Para.html#c:validate/2","title":"Para.validate/2","type":"callback"},{"doc":"Define a validator schema with an action name and field definitions. This will generate a new function called validate/2 with the action name and params as the arguments. iex&gt; defmodule UserParams do ...&gt; use Para ...&gt; ...&gt; validator :create do ...&gt; required :name ...&gt; end ...&gt; end ...&gt; ...&gt; UserParams . validate ( :create , %{ &quot;name&quot; =&gt; &quot;Syamil MJ&quot; } ) { :ok , %{ name : &quot;Syamil MJ&quot; } }","ref":"Para.html#validator/2","title":"Para.validator/2","type":"macro"},{"doc":"","ref":"Para.html#t:data/0","title":"Para.data/0","type":"type"},{"doc":"","ref":"Para.html#t:spec/0","title":"Para.spec/0","type":"type"},{"doc":"","ref":"Para.html#t:t/0","title":"Para.t/0","type":"type"},{"doc":"CHANGELOG","ref":"changelog.html","title":"CHANGELOG","type":"extras"},{"doc":"Added spec/2 - helper callback function to build custom changesets Improvements Fixed documentation for custom inline validator","ref":"changelog.html#v0-3-0","title":"CHANGELOG - [v0.3.0]","type":"extras"},{"doc":"Improvements Extend support for custom inline validator","ref":"changelog.html#v0-2-4","title":"CHANGELOG - [v0.2.4]","type":"extras"},{"doc":"Improvements Added .formatter.exs to package","ref":"changelog.html#v0-2-3","title":"CHANGELOG - [v0.2.3]","type":"extras"},{"doc":"Improvements Fix embeds_many Fix return values for embedded map or array of maps Improve docs","ref":"changelog.html#v0-2-2","title":"CHANGELOG - [v0.2.2]","type":"extras"},{"doc":"Improvements Simplify spec defaults Refactor function organization Fixes embeds callback","ref":"changelog.html#v0-2-1","title":"CHANGELOG - [v0.2.1]","type":"extras"},{"doc":"Added embeds_one - support embedding a single map embeds_many - support embedding list of maps Note - You can always use {:map, :string} or {:array, :map} as field type if you don't care about validation for the embedded fields. Changed Use converted params to ensure consistency of map keys","ref":"changelog.html#v0-2-0","title":"CHANGELOG - [v0.2.0]","type":"extras"},{"doc":"Added Implement validator callback","ref":"changelog.html#v0-1-1","title":"CHANGELOG - [v0.1.1]","type":"extras"},{"doc":"Added all the things","ref":"changelog.html#v0-1-0","title":"CHANGELOG - [v0.1.0]","type":"extras"}]